from .common import DtoObject


class MiniSeriesDto(DtoObject):
    pass


class LeagueEntryDto(DtoObject):
    pass


class LeagueDto(DtoObject):
    pass


class LeagueSummonerEntriesDto(DtoObject):
    pass


class LeagueEntriesDto(DtoObject):
    pass


class ChallengerLeagueListDto(DtoObject):
    pass


class GrandmasterLeagueListDto(DtoObject):
    pass


class MasterLeagueListDto(DtoObject):
    pass
